/*
 *
 * Copyright 1999, Cadence Design Systems, Incorporated.
 *
 * Cadence Design Systems (Cadence) and the author hereby disclaim
 * all implied warranties, including the implied warranties of merchant-
 * ability and fitness for a particular purpose. Cadence is not liable for
 * any damages incurred by the use of this software and/or documentation,
 * including direct, indirect, special, incidental, or consequential
 * damages.
 *
 * Cadence grants the bearer the right to modify, copy, and redistribute
 * this software and documentation, but requests that Cadence be acknowledged
 * for its contribution in the resulting product.
 *
 */

import java.util.Vector;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.io.*;

/**
 * A class for reading and writing the SpiTran project files. This also calls
 * registered listeners that need to load additional data files for the project.
 *
 * @author Anish Malhotra, Cadence Design Systems
 * @see s2iDBOpenListener
 */
public class s2iDBReadWrite{

/**
 * List of registered DB open listeners
 * @see s2iDBOpenListener
 */
  static java.util.Vector s2iDBOpenListeners = new Vector();

/**
 * Register a DB open listener to be called on DB open.
 * @see s2iDBOpenListener
 */
   public static void addDBOpenListener(s2iDBOpenListener l)
   {
      s2iDBOpenListeners.addElement(l);
   }

/**
 * Unregister a DB open listener.
 * @see s2iDBOpenListener
 */
   public static void removeDBOpenListener(s2iDBOpenListener l)
   {
      s2iDBOpenListeners.removeElement(l);
   }

/**
 * Call the getDB() methods of the registered DB open listeners.
 * @see s2iDBOpenListener
 */
   public static void fireDBOpenEvent()
   {
      for (Enumeration e = s2iDBOpenListeners.elements(); e.hasMoreElements();)
      {
         s2iDBOpenListener l = (s2iDBOpenListener) e.nextElement();
         l.getDB();
      }
   }

/**
 * Save project data to the project file.
 */
  public static void writeDBtoFile()
  {
     // Open the file for writing.
     String DBFileName = s2iGlobals.ProjDirFile + java.io.File.separator +
                         s2iGlobals.ProjName + s2iGlobals.PROJ_FILE_EXTN;
     PrintWriter out;

     try
     {
        out = new PrintWriter(new BufferedWriter(new FileWriter(DBFileName)));
        String msg = "Project saved: " + DBFileName;
        s2iError.PrintMsg(msg, s2iError.INFO,
                          s2iError.LOG | s2iError.DISPLAY, null);
     }
     catch (java.io.FileNotFoundException FnotFnd)
     {
        String msg = "File not Found: " + FnotFnd.getMessage();
        s2iError.PrintMsg(msg, s2iError.ERR, s2iError.POPUP |
                            s2iError.LOG | s2iError.DISPLAY, null);
        return ;
     }
     catch (java.io.IOException ioe)
     {
        String msg = ioe.toString();
        s2iError.PrintMsg(msg, s2iError.ERR, s2iError.POPUP |
                            s2iError.LOG | s2iError.DISPLAY, null);
        return ;
     }


     /***********************
      * writing the Header. *
      ***********************/
     out.println("Version " + s2iGlobals.VERSION);
     out.println("");

     /**********************************
      * writing the Buffer Technology. *
      **********************************/
     out.println("Buffer-Technology " + s2iGlobals.gBufTech);
     out.println("");

     /**********************************
      * writing the SPICE File.        *
      **********************************/
     out.println("SPICE-File " + s2iGlobals.spiceFileName);
     out.println("");

     /****************************************
      * writing Libs-and-File LIBS_AND_FILES *
      ****************************************/

     out.println("TYP_LIBS_AND_FILES");
     int size = s2iGlobals.gTypLibTreeList.size();
     for (int i =0 ; i< size; i++)
     {
        out.println((String) s2iGlobals.gTypLibTreeList.elementAt(i));
     }
     out.println("END_TYP_LIBS_AND_FILES");
     out.println("");

     out.println("BEST_LIBS_AND_FILES");
     size = s2iGlobals.gBestLibTreeList.size();
     for (int i =0 ; i< size; i++)
     {
        out.println((String) s2iGlobals.gBestLibTreeList.elementAt(i));
     }
     out.println("END_BEST_LIBS_AND_FILES");
     out.println("");

     out.println("WORST_LIBS_AND_FILES");
     size = s2iGlobals.gWorstLibTreeList.size();
     for (int i =0 ; i< size; i++)
     {
        out.println((String) s2iGlobals.gWorstLibTreeList.elementAt(i));
     }
     out.println("END_WORST_LIBS_AND_FILES");
     out.println("");


     /******************************************************
      * writing Models and Files INCLUDED_MODELS_AND_FILES *
      ******************************************************/

     out.println("TYP_INCLUDED_MODELS_AND_FILES");
     size = s2iGlobals.gTypModelTreeList.size();
     for (int i =0 ; i< size; i++)
     {
        out.println((String) s2iGlobals.gTypModelTreeList.elementAt(i));
     }
     out.println("END_TYP_INCLUDED_MODELS_AND_FILES");
     out.println("");

     out.println("BEST_INCLUDED_MODELS_AND_FILES");
     size = s2iGlobals.gBestModelTreeList.size();
     for (int i =0 ; i< size; i++)
     {
        out.println((String) s2iGlobals.gBestModelTreeList.elementAt(i));
     }
     out.println("END_BEST_INCLUDED_MODELS_AND_FILES");
     out.println("");

     out.println("WORST_INCLUDED_MODELS_AND_FILES");
     size = s2iGlobals.gWorstModelTreeList.size();
     for (int i =0 ; i< size; i++)
     {
        out.println((String) s2iGlobals.gWorstModelTreeList.elementAt(i));
     }
     out.println("END_WORST_INCLUDED_MODELS_AND_FILES");
     out.println("");

     /**************************************************************
      * writing Instantiated Sub-circuits INSTANTIATED_SUBCIRCUITS *
      **************************************************************/
     out.println("INSTANTIATED_SUBCIRCUITS");
     size = s2iGlobals.gInstanceVector.size();
     for (int i =0 ; i< size; i++)
     {
        out.println((String) s2iGlobals.gInstanceVector.elementAt(i));
     }
     out.println("END_INSTANTIATED_SUBCIRCUITS");
     out.println("");

     /********************************************
      * writing configuration nodes CONFIG_NODES *
      ********************************************/
     out.println("CONFIG_NODES");
     size = s2iGlobals.gConfigVector.size();
     for (int i =0 ; i< size; i++)
     {
        out.println((String) s2iGlobals.gConfigVector.elementAt(i));
     }
     out.println("END_CONFIG_NODES");
     out.println("");

     // Close the file.
     out.close();
  }

/**
 * Determine if the version of a SpiTran project file matches the current
 * version. This should be called after reading a "Version" token from the
 * file. It will consume tokens until the version is known. Nothing is
 * printed if version matching fails.
 * @param token The "Version" token String.
 * @param tokenizer A StringTokenizer for reading more tokens.
 * @return true if the version is correct.
 */
  private static boolean isVersionOK(String token, StringTokenizer tokenizer)
  {
     if (token.equals("Version"))
     {
        if (tokenizer.hasMoreTokens())
        {
           token = tokenizer.nextToken();
           double version;
           try
           {
              version = (new java.lang.Double(token)).doubleValue();
              if (s2iGlobals.VERSION != version)
                 return(false); //Later we may uprev here.
           }
           catch (java.lang.NumberFormatException nfe)
           {
              // Version unreadable
              return(false);
           }
        }
        else
        {
           // No version specified.
           return(false);
        }
     }
     else
     {
        // No version specified.
        return(false);
     }

     return(true);
  }

/**
 * Fill a Vector with lines from a file until a line beginning with a particular
 * token is seen.
 * @param vl The Vector to fill with line Strings.
 * @param fileBuf A BufferedReader for reading lines from the file.
 * @param endToken The token String that signifies end of data.
 */
  private static void fillVector(Vector v1, BufferedReader fileBuf, String endToken)
  {
     String line;
     v1.setSize(0);
     try
     {
        for (line = fileBuf.readLine(); line != null; line = fileBuf.readLine())
        {
           s2iError.PrintMsg(line, s2iError.DEBUG,
                            s2iError.LOG | s2iError.DISPLAY, null);
           if (line.startsWith(endToken))
              return;
           v1.addElement(line);
        }
     }
     catch (java.io.IOException ioe)
     {
        String Msg = ioe.toString();
        s2iError.PrintMsg(Msg, s2iError.ERR, s2iError.POPUP |
                            s2iError.LOG | s2iError.DISPLAY, null);
        return ;
     }
     return;
  }

/**
 * Read data from a SpiTran project file. Data is stored on the static
 * field members in s2iGlobals.
 * @param fileName The path String of the project file to read.
 * @see s2iGlobals
 */
  public static void readDBfromFile(String fileName)
  {
     BufferedReader inFile = null;
     String line;
     String fstTok, token;
     StringTokenizer tokenizer;

     // Open the file for reading.
     try
     {
        inFile = new BufferedReader(new FileReader(fileName));
     }
     catch (java.io.FileNotFoundException fnfe)
     {
        String msg = "File not Found: " + fnfe.getMessage();
        s2iError.PrintMsg(msg, s2iError.ERR, s2iError.POPUP |
                            s2iError.LOG | s2iError.DISPLAY, null);
        return ;
     }

     {
        s2iGlobals.ProjDirFile = new File(new File(fileName).getParent());

        String FileNameWExt = (new File(fileName)).getName();
        int len = FileNameWExt.length();
        int baseFileNameEnd = FileNameWExt.lastIndexOf(".");
        if (baseFileNameEnd == -1) baseFileNameEnd =len-1;
        s2iGlobals.ProjName = FileNameWExt.substring(0, baseFileNameEnd);

        s2iError.PrintMsg(s2iGlobals.ProjDirFile.toString(), s2iError.DEBUG,
                            s2iError.LOG | s2iError.DISPLAY, null);
        s2iError.PrintMsg(s2iGlobals.ProjName, s2iError.DEBUG,
                            s2iError.LOG | s2iError.DISPLAY, null);
     }

     try
     {
        line = inFile.readLine();

        /***********************************************
         * Before anything else, check for the version *
         ***********************************************/
        if (line != null)
        {
           s2iError.PrintMsg(line, s2iError.DEBUG,
                             s2iError.LOG | s2iError.DISPLAY, null);
           fstTok = null; token = null;
           tokenizer = new StringTokenizer(line);
           if (tokenizer.hasMoreTokens())
              fstTok = tokenizer.nextToken();

           if (fstTok == null)
              return;
           if ( !isVersionOK(fstTok, tokenizer))
              return;
        }

        for (line = inFile.readLine(); line != null; line = inFile.readLine())
        {
           s2iError.PrintMsg(line, s2iError.DEBUG,
                             s2iError.LOG | s2iError.DISPLAY, null);
           fstTok = null; token = null;
           tokenizer = new StringTokenizer(line);
           if (tokenizer.hasMoreTokens())
              fstTok = tokenizer.nextToken();

           if (fstTok == null)
           {
              continue;
           }
           if (fstTok.equals("Buffer-Technology"))
           {
              if (tokenizer.hasMoreTokens())
                 s2iGlobals.gBufTech = tokenizer.nextToken(); 
           }
           else if (fstTok.equals("SPICE-File"))
           {
              if (tokenizer.hasMoreTokens())
                 s2iGlobals.spiceFileName = tokenizer.nextToken(); 
           }
           else if (fstTok.equals("TYP_LIBS_AND_FILES"))
           {
              if (s2iGlobals.gTypLibTreeList == null)
                 s2iGlobals.gTypLibTreeList = new Vector();
              fillVector(s2iGlobals.gTypLibTreeList, inFile, "END_TYP_LIBS_AND_FILES");
           }
           else if (fstTok.equals("BEST_LIBS_AND_FILES"))
           {
              if (s2iGlobals.gBestLibTreeList == null)
                 s2iGlobals.gBestLibTreeList = new Vector();
              fillVector(s2iGlobals.gBestLibTreeList, inFile, "END_BEST_LIBS_AND_FILES");
           }
           else if (fstTok.equals("WORST_LIBS_AND_FILES"))
           {
              if (s2iGlobals.gWorstLibTreeList == null)
                 s2iGlobals.gWorstLibTreeList = new Vector();
              fillVector(s2iGlobals.gWorstLibTreeList, inFile, "END_WORST_LIBS_AND_FILES");
           }
           else if (fstTok.equals("TYP_INCLUDED_MODELS_AND_FILES"))
           {
              if (s2iGlobals.gTypModelTreeList == null)
                 s2iGlobals.gTypModelTreeList = new Vector();
              fillVector(s2iGlobals.gTypModelTreeList, inFile, "END_TYP_INCLUDED_MODELS_AND_FILES");
           }
           else if (fstTok.equals("BEST_INCLUDED_MODELS_AND_FILES"))
           {
              if (s2iGlobals.gBestModelTreeList == null)
                 s2iGlobals.gBestModelTreeList = new Vector();
              fillVector(s2iGlobals.gBestModelTreeList, inFile, "END_BEST_INCLUDED_MODELS_AND_FILES");
           }
           else if (fstTok.equals("WORST_INCLUDED_MODELS_AND_FILES"))
           {
              if (s2iGlobals.gWorstModelTreeList == null)
                 s2iGlobals.gWorstModelTreeList = new Vector();
              fillVector(s2iGlobals.gWorstModelTreeList, inFile, "END_WORST_INCLUDED_MODELS_AND_FILES");
           }
           else if (fstTok.equals("INSTANTIATED_SUBCIRCUITS"))
           {
              if (s2iGlobals.gInstanceVector == null)
                 s2iGlobals.gInstanceVector = new Vector();
              fillVector(s2iGlobals.gInstanceVector, inFile, "END_INSTANTIATED_SUBCIRCUITS");
           }
           else if (fstTok.equals("CONFIG_NODES"))
           {
              if (s2iGlobals.gConfigVector == null)
                 s2iGlobals.gConfigVector = new Vector();
              fillVector(s2iGlobals.gConfigVector, inFile, "END_CONFIG_NODES");
           }
        }
        inFile.close();
     }
     catch (java.io.IOException ioe)
     {
        String Msg = ioe.toString();
        s2iError.PrintMsg(Msg, s2iError.ERR, s2iError.POPUP | 
                            s2iError.LOG | s2iError.DISPLAY, null);
     }

     // Fire DB open event.
     fireDBOpenEvent();
  }


/**
 * Save data other than the project file. This complements the fireDBOpenEvent()
 * method. Currently this does nothing.
 */
  public static void fireDBSaveEvent()
  {
     /* This would be generating the files (Instantiation, config nodes, ..) 
      * from the data present in the DB.
      */
  }

}
