/*
 *
 * Copyright 1999, Cadence Design Systems, Incorporated.
 *
 * Cadence Design Systems (Cadence) and the author hereby disclaim
 * all implied warranties, including the implied warranties of merchant-
 * ability and fitness for a particular purpose. Cadence is not liable for
 * any damages incurred by the use of this software and/or documentation,
 * including direct, indirect, special, incidental, or consequential
 * damages.
 *
 * Cadence grants the bearer the right to modify, copy, and redistribute
 * this software and documentation, but requests that Cadence be acknowledged
 * for its contribution in the resulting product.
 *
 */

/**
 * The s2iData class holds all of the data that an s2ibis input file (.s2i)
 * will contain. Other classes read, write, and edit this data.
 *
 * @author Anish Malhotra, Cadence Design Systems
 * @see s2iReader
 * @see s2iWriter
 * @see s2iTransOptions
 */

public class s2iData extends Object
{
   /**
    * s2i output file
    */
   String  s2iFileName;

   /**
    * Header section parameter - version
    */
   String  ibsVersion;
   /**
   * Header section parameter - IBIS file name
   */
   String  ibsFileName;
   /**
   * Header section parameter - file revision
   */
   String  ibsFileRev;
   /**
   * Header section parameter - date
   */
   String  ibsDate;
   /**
   * Header section parameter - source
   */
   String  ibsSource;
   /**
   * Header section parameter - notes
   */
   String  ibsNotes;
   /**
   * Header section parameter - disclaimer
   */
   String  ibsDisclaimer;
   /**
   * Header section parameter - copyright
   */
   String  ibsCopyright;
   /**
   * Header section parameter - spice type
   */
   String  ibsSpiceType;
   /**
   * Header section parameter - spice command
   */
   String  ibsSpiceComm;
   /**
   * Header section parameter - iterate flag
   */
   boolean  ibsIterate = false;
   /**
   * Header section parameter - cleanup flag
   */
   boolean  ibsCleanup = false;
   /**
   * Header section parameter - summarize flag
   */
   boolean  ibsSummarize = false;


   // Where are fields for "Temperature range" to "Derate ramp" ?


   /**
   * Component section parameter - component name
   */
   String  ibsComponent;
   /**
   * Component section parameter - manufacturer
   */
   String  ibsManufacturer;
   /**
   * Component section parameter - package name
   */
   String  ibsPackageName;
   /**
   * Component section parameter - spice file name
   */
   String  ibsSpiceFile;

   /* ******************************************************************
    * We need not create multiple copies of parameters for components  *
    * and models, as we are not supporting multi-models/components.    *
    * We'll simply go for one.                                         *
    ********************************************************************/

    // parameters
   /**
   * Global parameter section parameter - typ package resistance
   */
   String  ibsRpkgtyp;
   /**
    * Global parameter section parameter - min package resistance
   */
   String  ibsRpkgmin;
   /**
   * Global parameter section parameter - max package resistance
   */
   String  ibsRpkgmax;
   /**
   * Global parameter section parameter - typ package inductance
   */
   String  ibsLpkgtyp;
   /**
   * Global parameter section parameter - min package inductance
   */
   String  ibsLpkgmin;
   /**
   * Global parameter section parameter - max package inductance
   */
   String  ibsLpkgmax;
   /**
   * Global parameter section parameter - typ package capacitance
   */
   String  ibsCpkgtyp;
   /**
    * Global parameter section parameter - min package capacitance
   */
   String  ibsCpkgmin;
   /**
   * Global parameter section parameter - max package capacitance
   */
   String  ibsCpkgmax;
   /**
   * Global parameter section parameter - typ package temperature
   */
   String  ibsTempTtyp;
   /**
   * Global parameter section parameter - min package temperature
   */
   String  ibsTempTmin;
   /**
   * Global parameter section parameter - max package temperature
   */
   String  ibsTempTmax;
   /**
   * Global parameter section parameter - typ supply voltage
   */
   String  ibsVoltVtyp;
   /**
   * Global parameter section parameter - min supply voltage
   */
   String  ibsVoltVmin;
   /**
   * Global parameter section parameter - max supply voltage
   */
   String  ibsVoltVmax;
   /**
   * Global parameter section parameter - typ pull-up voltage
   */
   String  ibsPullupVtyp;
   /**
   * Global parameter section parameter - min pull-up voltage
   */
   String  ibsPullupVmin;
   /**
    * Global parameter section parameter - max pull-up voltage
   */
   String  ibsPullupVmax;
   /**
   * Global parameter section parameter - typ pull-down voltage
   */
   String  ibsPulldnVtyp;
   /**
   * Global parameter section parameter - min pull-down voltage
   */
   String  ibsPulldnVmin;
   /**
   * Global parameter section parameter - max pull-down voltage
   */
   String  ibsPulldnVmax;
   /**
   * Global parameter section parameter - typ power-clamp voltage
   */
   String  ibsPclampVtyp;
   /**
   * Global parameter section parameter - min power-clamp voltage
   */
   String  ibsPclampVmin;
   /**
   * Global parameter section parameter - max power-clamp voltage
   */
   String  ibsPclampVmax;
   /**
   * Global parameter section parameter - typ ground-clamp voltage
   */
   String  ibsGclampVtyp;
   /**
   * Global parameter section parameter - min ground-clamp voltage
   */
   String  ibsGclampVmin;
   /**
   * Global parameter section parameter - max ground-clamp voltage
   */
   String  ibsGclampVmax;
   /**
   * Global parameter section parameter - typ silicon die capacitance
   */
   String  ibsCcompCtyp;
   /**
   * Global parameter section parameter - min silicon die capacitance
   */
   String  ibsCcompCmin;
   /**
   * Global parameter section parameter - max silicon die capacitance
   */
   String  ibsCcompCmax;
   /**
   * Global parameter section parameter - typ low stimulus input voltage
   */
   String  ibsVilVtyp;
   /**
   * Global parameter section parameter - min low stimulus input voltage
   */
   String  ibsVilVmin;
   /**
   * Global parameter section parameter - max low stimulus input voltage
   */
   String  ibsVilVmax;
   /**
   * Global parameter section parameter - typ high stimulus input voltage
   */
   String  ibsVihVtyp;
   /**
   * Global parameter section parameter - min high stimulus input voltage
   */
   String  ibsVihVmin;
   /**
   * Global parameter section parameter - max high stimulus input voltage
   */
   String  ibsVihVmax;
   /**
   * Global parameter section parameter - typ stimulus input voltage risetime
   */
   String  ibsTrTtyp;
   /**
   * Global parameter section parameter - min stimulus input voltage risetime
   */
   String  ibsTrTmin;
   /**
   * Global parameter section parameter - max stimulus input voltage risetime
   */
   String  ibsTrTmax;
   /**
   * Global parameter section parameter - typ stimulus input voltage falltime
   */
   String  ibsTfTtyp;
   /**
   * Global parameter section parameter - min stimulus input voltage falltime
   */
   String  ibsTfTmin;
   /**
   * Global parameter section parameter - max stimulus input voltage falltime
   */
   String  ibsTfTmax;

   /**
   * Global parameter section parameter - load resistance for ramp rate
   */
   String  ibsRload;
   /**
   * Global parameter section parameter - Spice transient simulation time
   */
   String  ibsSimTime;
   /**
   * Global parameter section parameter - threshold for clamp curve printing
   */
   String  ibsClampTol;
   /**
   * Global parameter section parameter - percent to derate VI curves
   */
   String  ibsDerateVi;
   /**
   * Global parameter section parameter - percent to derate ramp rates
   */
   String  ibsDerateRamp;



   /**
   * Model section parameter - model name
   */
   String  ibsModelName;
   /**
   * Model section parameter - no model flag
   */
   String  ibsNoModel;
   /**
   * Model section parameter - model type
   */
   String  ibsModelType;
   /**
   * Model section parameter - polarity
   */
   String  ibsPolarity;
   /**
   * Model section parameter - enable flag
   */
   String  ibsEnable;

   /**
   * Model section parameter - voltage low vin
   */
   String  ibsVin;
   /**
   * Model section parameter - voltage high vin
   */
   String  ibsVinh;
   /**
   * Model section parameter - voltage mean
   */
   String  ibsVmeas;
   /**
   * Model section parameter - ref capacitance
   */
   String  ibsCref;
   /**
   * Model section parameter - ref resistance
   */
   String  ibsRref;
   /**
   * Model section parameter - ref voltage
   */
   String  ibsVref;
   /**
   * Model section parameter - ground typ resistance
   */
   String  ibsRgndRtyp;
   /**
   * Model section parameter - ground min resistance
   */
   String  ibsRgndRmin;
   /**
   * Model section parameter - ground max resistance
   */
   String  ibsRgndRmax;
   /**
   * Model section parameter - power typ resistance
   */
   String  ibsRpowerRtyp;
   /**
   * Model section parameter - power min resistance
   */
   String  ibsRpowerRmin;
   /**
   * Model section parameter - power max resistance
   */
   String  ibsRpowerRmax;
   /**
   * Model section parameter - power typ Rac resistance
   */
   String  ibsRacRtyp;
   /**
   * Model section parameter - power min Rac resistance
   */
   String  ibsRacRmin;
   /**
   * Model section parameter - power max Rac resistance
   */
   String  ibsRacRmax;
   /**
   * Model section parameter - power typ Cac capacitance
   */
   String  ibsCacCtyp;
   /**
   * Model section parameter - power min Cac capacitance
   */
   String  ibsCacCmin;
   /**
   * Model section parameter - power max Cac capacitance
   */
   String  ibsCacCmax;
   /**
   * Model section parameter - power typ model file
   */
   String  ibsModelFiletyp;
   /**
   * Model section parameter - power min model file
   */
   String  ibsModelFilemin;
   /**
   * Model section parameter - power max model file
   */
   String  ibsModelFilemax;

   /**
   * Number of Falling waveforms
   */
   int FwaveCtr = 0;
   /**
   * Number of Rising waveforms
   */
   int RwaveCtr = 0;
   /**
   * Model section parameter - rising-wave Rf
   */
   String  ibsRwaveRf[] = new String[2];
   /**
   * Model section parameter - rising-wave Vf
   */
   String  ibsRwaveVf[] = new String[2];
   /**
   * Model section parameter - rising-wave Vf_min
   */
   String  ibsRwaveVfMin[] = new String[2];
   /**
   * Model section parameter - rising-wave Vf_max
   */
   String  ibsRwaveVfMax[] = new String[2];
   /**
   Model section parameter - rising-wave Lf
   */
   String  ibsRwaveLf[] = new String[2];
   /**
   * Model section parameter - rising-wave Cf
   */
   String  ibsRwaveCf[] = new String[2];
   /**
   * Model section parameter - rising-wave Rd
   */
   String  ibsRwaveRd[] = new String[2];
   /**
   * Model section parameter - rising-wave Cd
   */
   String  ibsRwaveCd[] = new String[2];
   /**
   * Model section parameter - rising-wave Ld
   */
   String  ibsRwaveLd[] = new String[2];
   /**
   * Model section parameter - falling-wave Rf
   */
   String  ibsFwaveRf[] = new String[2];
   /**
   * Model section parameter - falling-wave Vf
   */
   String  ibsFwaveVf[] = new String[2];
   /**
   * Model section parameter - falling-wave Vf_min
   */
   String  ibsFwaveVfMin[] = new String[2];
   /**
   * Model section parameter - falling-wave Vf_max
   */
   String  ibsFwaveVfMax[] = new String[2];
   /**
   * Model section parameter - falling-wave Lf
   */
   String  ibsFwaveLf[] = new String[2];
   /**
   * Model section parameter - falling-wave Cf
   */
   String  ibsFwaveCf[] = new String[2];
   /**
   * Model section parameter - falling-wave Rd
   */
   String  ibsFwaveRd[] = new String[2];
   /**
   * Model section parameter - falling-wave Cd
   */
   String  ibsFwaveCd[] = new String[2];
   /**
   * Model section parameter - falling-wave Ld
   */
   String  ibsFwaveLd[] = new String[2];

   // list of pins
   // java.awt.List ibsPinList;

   /**
   * The test fixture output pin node name.
   */
   String spice_node;

   /**
   * Name of the model.
   */
   String model_name;

   /**
   * The test fixture data input pin node name.
   */
   String input_pin;

   /**
   * The test fixture output enable pin node name.
   */
   String enable_pin;

   /**
   * The test fixture power pin node name.
   */
   String pow_pin;

   /**
   * The test fixture ground pin node name.
   */
   String gnd_pin;

}
