/*
 *
 * Copyright 1999, Cadence Design Systems, Incorporated.
 *
 * Cadence Design Systems (Cadence) and the author hereby disclaim
 * all implied warranties, including the implied warranties of merchant-
 * ability and fitness for a particular purpose. Cadence is not liable for
 * any damages incurred by the use of this software and/or documentation,
 * including direct, indirect, special, incidental, or consequential
 * damages.
 *
 * Cadence grants the bearer the right to modify, copy, and redistribute
 * this software and documentation, but requests that Cadence be acknowledged
 * for its contribution in the resulting product.
 *
 */

import java.util.Vector;

/**
 * A class for keeping SpiTran global variables. All variables are static
 * to the class. This class is never instantiated.
 *
 * @author Anish Malhotra, Cadence Design Systems
 */
public class s2iGlobals{

  /* *************************************************************************
   *
   *                  C O N S T A N T S
   *
   ***************************************************************************/

   /* SPICE file syntax related information */
   /**
   * Beginning of SpiTran identification record
   */
   public static final String ST_IDENT_REC = "Spi-Tran IDENTIFICATION RECORD BEGINS";
   /**
   * End of SpiTran identification record
   */
   public static final String EO_IDENT_REC = "Spi-Tran IDENTIFICATION RECORD ENDS";
   /**
   * Spice comment leader
   */
   public static final String ST_COMMENT = "*";
   /**
   * Spice inline comment leader
   */
   public static final String ST_INLINE_COMMENT = "$";
   /**
   * Spice continuation line leader
   */
   public static final String SPICE_PREV_LINE_CONT = "+";
   /**
   * Spice continuation line leader ???
   */
   public static final String SPICE_LINE_CONT = "\\\\";
   /**
   * Spice continuation line leader ???
   */
   public static final String SPICE_LINE_CONT_PSERV = "\\";



   /* Project File Related information */
   /**
   * File extension separator character
   */
   public static final char FILE_EXTN_SEP = '.';
   /**
   * File extension separator string
   */
   public static final String FILE_EXTN_SEP_STR = ".";
   /**
   * SpiTran project file extension
   */
   public static final String PROJ_FILE_EXTN = ".spp";
   /**
   * Prefix string for temporary file names
   */
   public static final String TMP_FILE_PREFIX = "#T";
   /**
   * Temporary file extension
   */
   public static final String TMP_FILE_SUFFIX = ".tmp";
   /**
   * s2ibis input file extension
   */
   public static final String S2I_CMD_FILE_EXTN = ".s2i";
   /**
   * ibis file extension
   */
   public static final String IBIS_FILE_EXTN = ".ibs";
   /**
   * s2ibis2 output save file
   */
   public static final String RUN_SAVE_FILE = "s2ibis2.txt";


   /* Property File information */
   /**
   * SpiTran properties file
   */
   public static final String propFileName = "SpiTran.ini";
   /**
   * Text editor command for Solaris
   */
   public static final String DEF_SOLARIS_EDITOR = "xterm -e vi";
   /**
   * Text editor command for HP PA-RISC
   */
   public static final String DEF_HP_EDITOR = "xterm -e vi";
   /**
   * Text editor command for AIX
   */
   public static final String DEF_IBM_EDITOR = "aixterm -bg white -e vi";
   /**
   * Text editor command for Windows
   */
   public static final String DEF_WIN_EDITOR = "write";

   /* Enumeration Type constants used all over the place */

   // Technologies
   /**
   * CMOS technology
   */
   public static final int CMOS = 1;
   /**
   * BJT technology
   */
   public static final int BJT = 2;
   /**
   * BICMOS technology
   */
   public static final int BICMOS = 3;
   /**
   * ECL technology
   */
   public static final int ECL = 4;

   //Keywords
   /**
   * Spice .INCLUDE keyword
   */
   public static final String KWS_INCLUDE = new String(".INCLUDE");
   /**
   * Spice .LIB keyword
   */
   public static final String KWS_LIB = new String(".LIB");
   /**
   * Spice .MODEL keyword
   */
   public static final String KWS_MODEL = new String(".MODEL");
   /**
   * Spice .SUBCKT keyword
   */
   public static final String KWS_SUBCKT = new String(".SUBCKT");
   /**
   * Spice .ENDS keyword
   */
   public static final String KWS_SUBCKT_END = new String(".ENDS");
   /**
   * Spice .GLBAL keyword
   */
   public static final String KWS_GLOBAL = new String(".GLOBAL");

   // Base names of nodes, fileNames being generated.
   /**
   * Prefix for SpiTran-generated Spice nodes
   */
   public static final String baseNodeName = "SpiNode";
   /**
   * Prefix for SpiTran-generated Spice R elements
   */
   public static final String baseRElemName = "RSpielem";
   /**
   * Prefix for SpiTran-generated Spice V elements
   */
   public static final String baseVElemName = "VSpielem";
   /**
   * SpiTran-generated typical model file name
   */
   public static final String TypModelFileName = "ModelTyp.sp";
   /**
   * SpiTran-generated minimum model file name
   */
   public static final String MinModelFileName = "ModelMin.sp";
   /**
   * SpiTran-generated maximum model file name
   */
   public static final String MaxModelFileName = "ModelMax.sp";
   /**
   * SpiTran-generated file name prefix
   */
   public static final String XFileNamePrefix = "X_";
   /**
   * SpiTran-generated typical configuration file name prefix
   */
   public static final String confTypFileNamePfx = "confTyp_";
   /**
   * SpiTran-generated minimum configuration file name prefix
   */
   public static final String confMinFileNamePfx = "confMin_";
   /**
   * SpiTran-generated maximum configuration file name prefix
   */
   public static final String confMaxFileNamePfx = "confMax_";

   // Commands and their options
   /**
   * s2ibis command string
   */
   public static final String s2ibisExe = "s2ibis2";

  /* *************************************************************************
   *
   *                  G L O B A L       V A R I A B L E S
   *
   ***************************************************************************/
  
   /**
   * The root directory of the SpiTran installation
   */
   public static String SpiTranRoot = null;
   /**
   * The current SpiTran project file
   */
   public static java.io.File ProjDirFile = null;
   /**
   * The current project directory path
   */
   public static String ProjName = null;
   /**
   * The current Spice file name
   */
   public static String spiceFileName = null;
   /**
   * The current Spice buffer technology type
   */
   public static String gBufTech = null;

   // Things that get written in the project file - Call it DB
   /**
   * The current SpiTran version number
   */
   public static final double VERSION = 1.0;

   /* Assign objects to the object references. Their being null causes
    * NullPointerException here and there in the code
    */
   /**
   * The current tree list of typical library files
   */
   public static Vector gTypLibTreeList = new Vector();
   /**
   * The current tree list of best-case library files
   */
   public static Vector gBestLibTreeList = new Vector();
   /**
   * The current tree list of worst-case library files
   */
   public static Vector gWorstLibTreeList = new Vector();
   /**
   * The current tree list of typical model files
   */
   public static Vector gTypModelTreeList = new Vector();
   /**
   * The current tree list of best-case model files
   */
   public static Vector gBestModelTreeList = new Vector();
   /**
   * The current tree list of worst-case model files
   */
   public static Vector gWorstModelTreeList = new Vector();
   /**
   * The current list of instantiated Spice subckts
   */
   public static Vector gInstanceVector = new Vector();
   /**
   * The current list of top-level node configuration settings
   */
   public static Vector gConfigVector = new Vector();

   public static String getRootDir()
   {
     if(s2iGlobals.SpiTranRoot == null)
       s2iGlobals.SpiTranRoot = System.getProperty("user.dir");

     return s2iGlobals.SpiTranRoot;
   }

   public static String getImageDir()
   {
     return getRootDir() + java.io.File.separator + "data";
   }
}
