/*
 *
 * Copyright 1999, Cadence Design Systems, Incorporated.
 *
 * Cadence Design Systems (Cadence) and the author hereby disclaim
 * all implied warranties, including the implied warranties of merchant-
 * ability and fitness for a particular purpose. Cadence is not liable for
 * any damages incurred by the use of this software and/or documentation,
 * including direct, indirect, special, incidental, or consequential
 * damages.
 *
 * Cadence grants the bearer the right to modify, copy, and redistribute
 * this software and documentation, but requests that Cadence be acknowledged
 * for its contribution in the resulting product.
 *
 */


/* package COM.cadence.spi_tran; */

import java.util.StringTokenizer;
import java.util.Vector;
import java.util.Properties;
import java.util.Enumeration;
import java.io.*;

/** s2iSpiceReader will scan the Spice file specified as an argument of main().
    If the file has a Spi-Tran IDENTIFICATION RECORD then information from
    it will be stored in the SpiceFileInfo data-structure. This data-structure
    will be later used to populate the Spi-tran UI
    If there is no IDENTIFICATION RECORD, then the file will be cleaned.
*/
public class s2iSpiceReader{

   final String fileName;
   FileReader reader;
   BufferedReader buf_reader;
   FileWriter writer;
   BufferedWriter buf_writer = null;
   StringBuffer curLineBuff = new StringBuffer();
   String nxtLine = null;
     
   public s2iSpiceReader(String inFile) throws java.io.FileNotFoundException
   {
      fileName = inFile;
      reader = new FileReader(inFile);
      buf_reader = new BufferedReader(reader);
   }

   public s2iSpiceReader(String inFile, String outFile) 
        throws java.io.FileNotFoundException, java.io.IOException
   {
      fileName = inFile;
      reader = new FileReader(inFile);
      buf_reader = new BufferedReader(reader);
      writer = new FileWriter(outFile);
      buf_writer = new BufferedWriter(writer);
   }

   /*removes any whitespace characters present in the front of the string
   */
   private static StringBuffer frontTrim(String string)
   {
      StringTokenizer st = new StringTokenizer(string);
      String fstToken = null;
      StringBuffer retVal=null;

      if (st.hasMoreTokens()) fstToken=st.nextToken();
      if (fstToken != null)
      {
        int index = string.indexOf(fstToken);
        retVal = new StringBuffer(string.substring(index));
      } 

      return(retVal);
   }

   private static void joinSpiceLine(StringBuffer mainLine, String addLine)
   {
      int index, replaceLen;
      String fstPart, ScdPart;
      String mainLineStr = new String(mainLine.toString());
      String mainLineOrg = new String(mainLine.toString());
      StringBuffer addLineBuff = new StringBuffer(addLine);

      mainLineStr.trim();
      // The if-else-if block removes the unwanted characters
      // Check if mainLine ends with a spl meaning string
      if (mainLineStr.endsWith(s2iGlobals.SPICE_LINE_CONT))
      {
         //squeeze out any white space between the continued lines.
         index = mainLineOrg.lastIndexOf(s2iGlobals.SPICE_LINE_CONT);
         replaceLen = s2iGlobals.SPICE_LINE_CONT.length();
	 fstPart = mainLineOrg.substring( 0, index);
	 ScdPart = mainLineOrg.substring( 0 + index + replaceLen);
         fstPart = fstPart.trim();
         mainLine.setLength(0);
	 mainLine.append(fstPart);
         addLineBuff = frontTrim(addLine);
         
      }
      else if (mainLineStr.endsWith(s2iGlobals.SPICE_LINE_CONT_PSERV))
      {
         //preserve whitespace
         index = mainLineOrg.lastIndexOf(s2iGlobals.SPICE_LINE_CONT_PSERV);
         replaceLen = s2iGlobals.SPICE_LINE_CONT_PSERV.length();
	 fstPart = mainLineOrg.substring( 0 , index);
	 ScdPart = mainLineOrg.substring( 0 + index + replaceLen);
         mainLine.setLength(0);
	 mainLine.append(fstPart);
	 mainLine.append(ScdPart);

      }
      // Check if addLine starts with a spl meaning string
      else if (addLine.startsWith(s2iGlobals.SPICE_PREV_LINE_CONT))
      {

         //preserve whitespace
	 index = addLine.indexOf(s2iGlobals.SPICE_PREV_LINE_CONT);
	 replaceLen = s2iGlobals.SPICE_PREV_LINE_CONT.length();
	 fstPart = addLine.substring( 0 , index);
	 ScdPart = addLine.substring( 0 + index + replaceLen);
	 addLineBuff.setLength(0);
         addLineBuff.append(fstPart);
         addLineBuff.append(ScdPart);
         
      }


      //Now, simply join the two
      mainLine.append(addLineBuff);
   }


   /* METHODS */
   /** Gets one Spice Line from the Spice file. One Spice Line may continue
       over several lines with the use of '+' in the begining of a line.

       @param: Line  will return the SpiceFile Line. 
               AllLines returns the Lines on "as-is" basis which are comprised
               in SpiceFile Line.
       @return: returns the number of Lines of the Spice-File that make the \
                Spice File Line.
   */

   public int getSpiceLine(StringBuffer Line, Vector AllLines)
          throws java.io.IOException
   { int count = 0;
     String trimNxtLine=null, trimCurLine; 

     /*Reset the Line and AllLines */
     Line.setLength(0);
     AllLines.setSize(0);

     /* If we already have a next-line, read once; else read twice */
     curLineBuff.setLength(0);
     if (nxtLine != null)
     {
        curLineBuff.append(nxtLine);
     }
     else
     {
        nxtLine = buf_reader.readLine();
        if (nxtLine == null) 
            return(count);
        curLineBuff.append(nxtLine);
     }
     nxtLine = buf_reader.readLine();

     // Strip off commented out characters
     if(nxtLine != null)
     {
       int commentStart = nxtLine.indexOf(s2iGlobals.ST_INLINE_COMMENT);
       int commentStart2 = nxtLine.indexOf(s2iGlobals.ST_COMMENT);
       if(commentStart < 0 || (commentStart2 >=0 && commentStart2 < commentStart))
         commentStart = commentStart2;
       if(commentStart >= 0)
        nxtLine = nxtLine.substring(0, commentStart);
      }
      
     /* Here, don't return if nxtLine is null <= curLineBuff is still
        unrecorded.
     */

     /*Add the contents of the lines */
     AllLines.addElement(
              new StringBuffer(curLineBuff.toString())); //add an object
     // processs Line
     joinSpiceLine(Line, curLineBuff.toString()); count++;

     trimCurLine = curLineBuff.toString();
     trimCurLine.trim();
     if (nxtLine != null)
     {
        trimNxtLine = nxtLine.toString();
        trimNxtLine.trim();
     }

     /* Keep on reading till the line is completed. A Line may continue
        onto the next Line if SPICE_PREV_LINE_CONT is present at SOL of
        next Line or SPICE_LINE_CONT* is present at EOL of current Line
     */ 
     while (trimCurLine.endsWith(s2iGlobals.SPICE_LINE_CONT) ||
            trimCurLine.endsWith(s2iGlobals.SPICE_LINE_CONT_PSERV) ||
            ( (trimNxtLine != null) && 
             trimNxtLine.startsWith(s2iGlobals.SPICE_PREV_LINE_CONT)))
     {
        /* Move the next-line into current line. And read the new
           next line.
        */
        curLineBuff.setLength(0);
        curLineBuff.append(nxtLine);
        nxtLine = buf_reader.readLine();

        if (nxtLine == null)
        {
           if (trimCurLine.endsWith(s2iGlobals.SPICE_LINE_CONT) ||
               trimCurLine.endsWith(s2iGlobals.SPICE_LINE_CONT_PSERV))
               s2iError.PrintMsg("Unexpected EOF", s2iError.ERR,
                                 s2iError.LOG | s2iError.DISPLAY, null);
           break ;
        }
        else
        {
           trimNxtLine = nxtLine.toString(); 
           trimNxtLine.trim(); //Trims leading and trailing whitespaces
        }
        AllLines.addElement(
                 new StringBuffer(curLineBuff.toString())); //add an object
        // processs Line 
	joinSpiceLine(Line, curLineBuff.toString()); count++;

        // Trim the current Line (NxtLine already done).
        trimCurLine = curLineBuff.toString();
        trimCurLine.trim();

      } //while

DONE:
      // Write out the SPICE line to an output file
      if (buf_writer != null)
      {
          buf_writer.write(Line.toString());
          buf_writer.newLine();
      }
      // return the number of Lines used up in making one SPICE Line
      return(count);
   }

   protected void close() throws java.io.IOException
   {
      if (buf_reader != null) buf_reader.close();
      if (buf_writer != null) buf_writer.close();
   }

   protected void finalize() throws Throwable
   {
      super.finalize();
      if (buf_reader != null) buf_reader.close();
      if (buf_writer != null) buf_writer.close();
   }

/***************************************************************************
 * This function is a static. It is to be passed a SPICE file as an argument.
 * It will return all the Included files in a string enumeration.
 ****************************************************************************/

   public static Enumeration getIncludeFiles(String fileWPath)
   throws java.io.FileNotFoundException, java.io.IOException
   {
      s2iSpiceReader fileReader = new s2iSpiceReader(fileWPath);
      Vector IncludeNames = new Vector();
      StringTokenizer tokens;
      String word, name;
      StringBuffer SpiceLine = new StringBuffer();
      Vector dummy = new Vector();

      while( fileReader.getSpiceLine(SpiceLine, dummy) != 0)
      {
         //Tokenize and look for a line starting with .Include
         tokens = new StringTokenizer(SpiceLine.toString());
         if (tokens.hasMoreTokens())
         {
            word = tokens.nextToken();
            if (word.equalsIgnoreCase(s2iGlobals.KWS_INCLUDE))
            {
               if (tokens.hasMoreTokens())
                  name = tokens.nextToken();
               else
                 name = null;
              if (name != null)
                 IncludeNames.addElement(name);
            }
         }
      }
      return(IncludeNames.elements());
   }

/***************************************************************************
 * This function is a static. It is to be passed a SPICE file as an argument.
 * It will return all the Sub-circuit Names in a string enumeration.
 ****************************************************************************/

   public static Enumeration getSubckts(String fileWPath)
   throws java.io.FileNotFoundException, java.io.IOException
   {
      s2iSpiceReader fileReader = new s2iSpiceReader(fileWPath);
      Vector subCktNames = new Vector();
      StringTokenizer tokens;
      String word, name;
      StringBuffer SpiceLine = new StringBuffer();
      Vector dummy = new Vector();

      while( fileReader.getSpiceLine(SpiceLine, dummy) != 0)
      {
         //Tokenize and look for a line starting with .Include
         tokens = new StringTokenizer(SpiceLine.toString());
         if (tokens.hasMoreTokens())
         {
            word = tokens.nextToken();
            if (word.equalsIgnoreCase(s2iGlobals.KWS_SUBCKT))
            {
               if (tokens.hasMoreTokens())
                  name = tokens.nextToken();
               else
                 name = null;
              if (name != null)
                 subCktNames.addElement(name);
            }
         }
      }
      return(subCktNames.elements());
   }


/***************************************************************************
 * This function is a static. It is to be passed a SPICE file as an argument.
 * It will return all the Sub-circuit definitions in a string enumeration.
 * anish - 27 Jan 99.
 ****************************************************************************/

   public static Enumeration getSubcktDefs(String fileWPath)
   throws java.io.FileNotFoundException, java.io.IOException
   {
      s2iSpiceReader fileReader = new s2iSpiceReader(fileWPath);
      Vector subCktDefs = new Vector();
      StringTokenizer tokens;
      String word; 
      StringBuffer name = new StringBuffer();
      StringBuffer SpiceLine = new StringBuffer();
      Vector dummy = new Vector();

      while( fileReader.getSpiceLine(SpiceLine, dummy) != 0)
      {
         //Tokenize and look for a line starting with .Include
         tokens = new StringTokenizer(SpiceLine.toString());
         if (tokens.hasMoreTokens())
         {
            word = tokens.nextToken();
            if (word.equalsIgnoreCase(s2iGlobals.KWS_SUBCKT))
            {
               name.setLength(0); 
               while (tokens.hasMoreTokens())
                  name.append(tokens.nextToken() + "  ");
               if (name.length() != 0)
                 subCktDefs.addElement(name.toString().trim());
            }
         }
      }
      return(subCktDefs.elements());
   }


/***************************************************************************
 * This function is a static. It is to be passed a SPICE file as an argument.
 * It checks the existence of all the LIBS and INCLUDES that have been 
 * referred to from this file. It is to be passed two vectors as arguments
 * which will store the value of the files not present.  It returns a true
 * if all files are existing, and a false otherwise or in case of an error
 * anish 01 Dec '98
 ****************************************************************************/
public static boolean allLnI_Exist(
                      String fileName, Vector misLibs, Vector misIncl)
   throws java.io.FileNotFoundException, java.io.IOException
{
   boolean allExist = true;
   File spiceFile = new File(fileName);

   if(!spiceFile.exists())
   {
      allExist = false;
   }

   if (allExist)
   /* ==> the file passed as an argument exists, so start looking into it */
   {
      s2iSpiceReader fileReader = new s2iSpiceReader(fileName);
      StringBuffer SpiceLine = new StringBuffer();
      String parent = new String(spiceFile.getParent());
      Vector dummy = new Vector();

      while( fileReader.getSpiceLine(SpiceLine, dummy) != 0)
      {
         StringTokenizer tokens = new StringTokenizer(SpiceLine.toString());
         String fst_word = null, scd_word = null, thd_word = null;
         if (tokens.hasMoreTokens())
         {
            fst_word = tokens.nextToken();
            if (tokens.hasMoreTokens()) scd_word = tokens.nextToken();
            if (tokens.hasMoreTokens()) thd_word = tokens.nextToken();
            boolean libReference = false;

            if (fst_word.equalsIgnoreCase(s2iGlobals.KWS_LIB))
            {
              /* A library call is of the form 
               * .LIB '<filepath>filename' entryname 
               * A Library definition statement is of the form
               * .LIB entryname
               * We are on the look for only a library call.
               */

               if (thd_word != null) libReference = true;
            }

            if ((fst_word.equalsIgnoreCase(s2iGlobals.KWS_INCLUDE) || 
                 libReference) &&
                 (scd_word != null))
            {
               boolean exist = true;
               /* Now we'll have another file to be checked for */
               /* So we get the file */
               StringTokenizer tokens2 = new StringTokenizer(scd_word,"'");
               if (tokens2.hasMoreTokens())
               {
                  String fileWPath = tokens2.nextToken();
                  String path = (new File(fileWPath)).getParent();
                  String fileWFullPath;
                  File LibOrInclFile;
                  /* Now look for the file. If the path is relative, append it 
                   * to the path of the spice file in which it has been referred
                   * i.e.. spiceFile
                   */
                  if (!(new File(path)).isAbsolute())
                  {
                     fileWFullPath = parent + fileWPath;
                  }
                  else
                  {
                     fileWFullPath = fileWPath;
                  }

                  /* Now check for this file's existence */
                  LibOrInclFile = new File(fileWFullPath);
                  if ( !LibOrInclFile.exists() || !LibOrInclFile.isFile() )
                  {
                     exist = false;
                     if (libReference)
                        misLibs.addElement(fileWFullPath);
                     else
                        misIncl.addElement(fileWFullPath);
                  }
                  else
                  {
                      exist = allLnI_Exist(fileWFullPath, misLibs, misIncl);
                  }
               } //  Checked the file referred in INCLUDE or LIB
               if (!exist) allExist = false;
            } // The word was .INCLUDE or .LIB
         } // The line had at least a token
      } // Till the time there are lines in the SPICE file
   } // If the SPICE file exists
   return(allExist);
}

/*  *************************************************************************
*/
   public static void findSpiceLineType(String SpiceLine)
   {
      String fstToken = null;
      StringTokenizer st = new StringTokenizer(SpiceLine);
      if (st.hasMoreTokens()) fstToken = st.nextToken();
   }


/*  *************************************************************************
    If the given file has an identification record then the file is read and
    relevant data is stored in structures. These structures will link the file
    to the UI.
    In abscence of an identification record a new file gets created.
*/
   public static void main(String args[]) 
   {
      StringBuffer SpiceLine = new StringBuffer();
      Vector componentLines = new Vector(1, 0);
      FileWriter writer;
      
      if ((args.length != 1) && (args.length != 2)) 
      {
         System.err.println("Usage: s2iSpiceReader <SPICE filename> [<output FileName>]");
         System.err.println("Property SPI_FILERDR_DBG allows printing a detailed output on stdout. [Optional]\n");
	 System.exit(1);
      }

      s2iSpiceReader SpiceReader=null; 
      String propVal = System.getProperty("SPI_FILERDR_DBG");
      String in_file = args[0];
      String out_file;
      try
      {
         if (args.length == 1) 
         {
             SpiceReader = new s2iSpiceReader(in_file);
         }
         else
         {
             out_file = args[1];
             SpiceReader = new s2iSpiceReader(in_file, out_file);
         }
      }
      catch (java.io.IOException ex)
      {
          s2iError.PrintMsg(ex.getMessage(), s2iError.ERR,
                            s2iError.POPUP | s2iError.LOG | s2iError.DISPLAY,
                            null);
      }
      

      try
      {
         /* This OUTPUTS the SPICE File after deciphering the line continuation
          * characters
          */
	 while ((SpiceReader.getSpiceLine(SpiceLine, componentLines)) != 0)
	 {
	    //Fill up the data-structures from the SpiceLine

	    // For debugging
	    if (propVal != null)
	       CheckBydump(SpiceLine, componentLines);
         }
         SpiceReader.close();


         /* Outputs the Include files and Subckts present in the given spice 
          * file.
          */
         {
            Enumeration Includes = s2iSpiceReader.getIncludeFiles(in_file);
            Enumeration subCkts = s2iSpiceReader.getSubckts(in_file);
            String tmpString;

            s2iError.Debug("The INCLUDEs are: ");
            boolean allExist = true;
            Vector missingLibs = new Vector();
            Vector missingIncl = new Vector();
            while (Includes.hasMoreElements())
            {
               tmpString = (String) Includes.nextElement();
               s2iError.Debug("   " + tmpString);
               /* Time to check for the exitence of all the Includes and
                * the LIBS and INCLUDES further in the depth
                */
               boolean exist = true;
               if ((new File(tmpString)).exists() && 
                   (new File(tmpString)).isFile())
               {
                  exist = allLnI_Exist(tmpString, missingLibs, missingIncl);
               }
               else
               {
                  exist = false;
                  missingIncl.addElement(tmpString);
               }
               if (!exist) allExist = false;
            }
            s2iError.Debug("The SUBCKTs are: ");
            while (subCkts.hasMoreElements())
            {
               tmpString = (String) subCkts.nextElement();
               s2iError.Debug("   " + tmpString);
            } 

            /**********************************************************
             * If some missing Libs and Includes have been found, list
             * those
             ********************************************************/
            if (!allExist)
            {
               s2iError.Debug("Missing LIBS/INCLUDES detected");
               s2iError.Debug("Note that any relative paths in the imported SPICE");
               s2iError.Debug("File, may have to be updated with respect to the project directory");
               if (!missingLibs.isEmpty())
               {
                  int size = missingLibs.size();
                  s2iError.Debug("Missing LIBS");
                  s2iError.Debug("=============");
                  for (int i=0; i< size; i++)
                  {
                    String Missing = (String) missingLibs.elementAt(i);
                    s2iError.Debug(Missing);
                  }
               }

               if (!missingIncl.isEmpty())
               {
                  int size = missingIncl.size();
                  s2iError.Debug("Missing LIBS");
                  s2iError.Debug("=============");
                  for (int i=0; i< size; i++)
                  {
                    String Missing = (String) missingIncl.elementAt(i);
                    s2iError.Debug(Missing);
                  }

               }
            }
         }
         

      } //try
      catch (java.io.FileNotFoundException ex)
      {
         s2iError.PrintMsg("Error: File \"" + ex.getMessage() + "\" does not exist",
                            s2iError.ERR,
                            s2iError.POPUP | s2iError.LOG | s2iError.DISPLAY,
                            null);
      }
      catch (java.io.IOException ex)
      {
         s2iError.PrintMsg(ex.getMessage(),
                            s2iError.ERR,
                            s2iError.POPUP | s2iError.LOG | s2iError.DISPLAY,
                            null);
      }

      
               
   }


/*  *************************************************************************
    This is to be used to check the correct interpretation of HSPICE's Input
    Line Format
*/ 
   public static void CheckBydump(StringBuffer FullLine, Vector componentLines)
   {
      StringBuffer Line;
      int i;

      /*Print what we got */
      s2iError.Debug("The complete Line is:");
      s2iError.Debug("=====================");
      s2iError.Debug(FullLine + "\n********OVER****\n");
      s2iError.Debug("It comprises:");
      s2iError.Debug("=============");
      for (i = 0; i < componentLines.size(); i++)
      {
         Line = (StringBuffer) componentLines.elementAt(i);
         s2iError.Debug(i + ": " + Line); 
      }
      s2iError.Debug("\n");
   }

} //Class s2iSpiceReader
